from ..cookieconsent import CookieConsentViewlet
from icemac.addressbook.browser.interfaces import IFanstaticViewletManager
from icemac.addressbook.testing import Webdriver
from icemac.addressbook.testing import WebdriverPageObjectBase
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait
import mock
import os
import pytest
import zope.component


@zope.interface.implementer(IFanstaticViewletManager)
class FakeFanstaticViewletManager(object):
    """Fake FanstaticViewletManager."""


@zope.interface.implementer(zope.browser.interfaces.IBrowserView)
class FakeBrowserView(object):
    """Fake BrowserView."""


def test_cookieconsent__CookieConsentViewlet__update__1(
        address_book, RequestFactory):
    """It contains the data protection URL and link text if the URL is set."""
    viewlet = zope.component.getMultiAdapter(
        (address_book,
         RequestFactory(),
         FakeBrowserView(),
         FakeFanstaticViewletManager()),
        zope.viewlet.interfaces.IViewlet,
        name='CookieConsent')
    assert isinstance(viewlet, CookieConsentViewlet)
    data = {'AB_LINK_DATAPROTECTION_URL': 'https://datapro.example.com'}
    with mock.patch.dict(os.environ, data):
        viewlet.update()
        result = viewlet.render()
    assert '"href": "https://datapro.example.com"' in result
    assert '"link": "Read more."' in result


def test_cookieconsent__CookieConsentViewlet__update__2(
        address_book, RequestFactory):
    """It contains neither data protection URL nor link text if ...

    ... URL isn't set.
    """
    viewlet = zope.component.getMultiAdapter(
        (address_book,
         RequestFactory(),
         FakeBrowserView(),
         FakeFanstaticViewletManager()),
        zope.viewlet.interfaces.IViewlet,
        name='CookieConsent')
    viewlet.update()
    result = viewlet.render()
    assert '"href": null' in result
    assert '"link": null' in result


@pytest.fixture('function')
def po_cookieconsent_webdriver():
    """Webdriver page object for the cookie consent dialog."""
    class POCookieConsent(WebdriverPageObjectBase):
        paths = [
            'ADDRESS_BOOK_DEFAULT_URL',
        ]

        class Button(object):

            def __init__(self, driver, selector, expect_to_be_visible):
                self.driver = driver
                self.selector = selector
                self.wait = WebDriverWait(driver, 3)  # seconds
                self.el = driver.find_element_by_css_selector(selector)
                if expect_to_be_visible:
                    self.wait.until(expected_conditions.visibility_of(self.el))

            @property
            def is_displayed(self):
                return self.el.is_displayed()

            def click(self):
                self.el.click()
                self.wait.until(
                    expected_conditions.invisibility_of_element_located(
                        (By.CSS_SELECTOR, self.selector)))

        def _get_button(self, expect_to_be_visible):
            return self.Button(
                self._selenium,
                '.cc-compliance .cc-dismiss',
                expect_to_be_visible)

        @property
        def button_is_displayed(self):
            button = self._get_button(expect_to_be_visible=True)
            return button.is_displayed

        @property
        def button_is_not_displayed(self):
            button = self._get_button(expect_to_be_visible=False)
            return not button.is_displayed

        def button_click(self):
            button = self._get_button(expect_to_be_visible=True)
            button.click()

    Webdriver.attach(POCookieConsent, 'cookieconsent')
    yield
    Webdriver.detach(POCookieConsent, 'cookieconsent')


@pytest.mark.webdriver
def test_cookieconsent__CookieConsentViewlet__render__1_webdriver(address_book, po_cookieconsent_webdriver, webdriver):  # noqa E501
    """It allows to dismiss the cookie consent dialog."""
    cookieconsent = webdriver.cookieconsent
    webdriver.login('visitor', cookieconsent.ADDRESS_BOOK_DEFAULT_URL)
    assert cookieconsent.button_is_displayed
    # Clicking the button dismisses the dialog:
    cookieconsent.button_click()
    assert cookieconsent.button_is_not_displayed
    # It does not show up again on other pages:
    webdriver.open(cookieconsent.ADDRESS_BOOK_DEFAULT_URL)
    assert cookieconsent.button_is_not_displayed



from ..breadcrumb import IBreadcrumb, Breadcrumb
from zope.testbrowser.browser import HTTPError
import mock
import pytest
import zope.interface.verify


def test_breadcrumb__Breadcrumb__1(zcmlS):
    """It fulfills the interface definition."""
    breadcrumb = Breadcrumb(None, None)
    with mock.patch.object(breadcrumb, 'url'):
        with mock.patch('zope.traversing.api.getParent'):
            zope.interface.verify.verifyObject(IBreadcrumb, breadcrumb)


def test_breadcrumb__Breadcrumb____repr____1():
    """It renders a nice repr."""
    class MyBreadcrumb(Breadcrumb):
        title = u'my title'

    breadcrumb = MyBreadcrumb(None, None)
    assert ("<icemac.addressbook.browser.tests.test_breadcrumb.MyBreadcrumb:"
            " u'my title'>" == repr(breadcrumb))


def test_breadcrumb__NotFoundBreadcrumb__1(address_book, browser):
    """It renders a nice title in the breadcrumbs if a page cannot be found."""
    browser.login('visitor')
    with pytest.raises(HTTPError) as err:
        browser.open('http://localhost/I-do-not-exist')
    assert 'HTTP Error 404: Not Found' == str(err.value)
    assert '>Not Found<' in browser.contents



# Starting point of data serialization for full text index, not complete, yet
import six


def FieldSerializer(field, obj):
    """Serialize the value of a field to a unicode value."""
    return six.text_type(field.get(obj))


def FieldNoSerializer(field, obj):
    """Serialize the value of a field to the empty string."""
    return u''



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
import grokcore.component as grok
import icemac.addressbook.browser.base
import icemac.addressbook.browser.interfaces
import icemac.addressbook.browser.table
import z3c.layer.pagelet.interfaces
import zope.browser.interfaces
import zope.contentprovider.interfaces
import zope.contentprovider.provider
import zope.interface
import zope.location.interfaces
import zope.preference.interfaces
import zope.publisher.interfaces
import zope.schema
import zope.traversing.api

DO_NOT_SHOW = 'I do not want to show up as a breadcrumb!'


class IBreadcrumb(zope.interface.Interface):
    """A single breadcrumb which knows its parent."""

    show = zope.schema.Bool(title=u'Render this breadcrumb in front end?')
    title = zope.schema.TextLine(title=u'Title rendered in front end.')
    target_url = zope.schema.URI(
        title=u'Target URL or `None` for a text breadcrumb',
        required=False)
    parent = zope.schema.Object(
        zope.location.interfaces.ILocation,
        title=u'Object for the parent breadcrumb.',
        description=u'If `None`, no further parent exists.',
        required=False)

    def render():
        """Render the HTML of the breadcrumb."""


@zope.interface.implementer(IBreadcrumb)
class Breadcrumb(grok.MultiAdapter,
                 icemac.addressbook.browser.base.BaseView):
    """Base breadcrumb implementation."""

    grok.baseclass()
    grok.provides(IBreadcrumb)

    show = True
    template = ViewPageTemplateFile('breadcrumb.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def title(self):
        try:
            title = icemac.addressbook.interfaces.ITitle(self.context)
        except (zope.security.interfaces.ForbiddenAttribute,
                ValueError):  # pragma: no cover
            title = "{}: <unprintable title>".format(
                self.context.__class__.__name__)
        return title

    @property
    def target_url(self):
        return self.url(self.context)

    @property
    def parent(self):
        return zope.traversing.api.getParent(self.context)

    def __repr__(self):
        """Curstom representation for breadcrumbs."""
        return '<{0}: {1.title!r}>'.format(
            icemac.addressbook.utils.dotted_name(self.__class__), self)

    def render(self):
        return self.template()


class MasterdataChildBreadcrumb(Breadcrumb):
    """Breadcrumb those parent is `masterdata.html`."""

    grok.baseclass()

    @property
    def parent(self):
        return zope.component.getMultiAdapter(
            (icemac.addressbook.interfaces.IAddressBook(self.context),
             self.request),
            name='masterdata.html')


class ViewBreadcrumb(Breadcrumb):
    """Default breadcrumb implementation for views."""

    grok.adapts(
        icemac.addressbook.browser.base.FlashView,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    target_url = None

    @property
    def show(self):
        return self.title != DO_NOT_SHOW

    @property
    def title(self):
        try:
            title = self.context.title
        except AttributeError:
            title = None
        if title is None:
            # AttributeError above or not set:
            title = u'👉 missing title! 😱 {!r}'.format(self.context)
        return title


class TableBreadcrumb(ViewBreadcrumb):
    """View for views based on the `Table` class and its subclasses."""

    grok.adapts(
        icemac.addressbook.browser.table.Table,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)


class RootBreadcrumb(Breadcrumb):
    """Breadcrumb for the root folder: Hide it."""

    grok.adapts(
        zope.site.interfaces.IRootFolder,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    show = False


class ModelBreadcrumb(Breadcrumb):
    """Default breadcrumb implementation for model objects."""

    grok.adapts(
        zope.location.interfaces.ILocation,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)


class PersonEntityBreadcrumb(Breadcrumb):
    """Do not render addresses ansd numbers as breadcrumbs.

    They do not have a default view and their only view is the delete view
    which renders the relevant information.
    """

    grok.adapts(
        icemac.addressbook.interfaces.IPersonEntity,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    show = False


class PreferenceCategoryBreadcrumb(Breadcrumb):
    """Breadcrumb for preference categories."""

    grok.adapts(
        zope.preference.interfaces.IPreferenceCategory,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    title = _('Preferences')

    @property
    def parent(self):
        return icemac.addressbook.interfaces.IAddressBook(self.context)

    @property
    def target_url(self):
        return self.url(self.parent, '++preferences++/ab')


class PreferenceGroupBreadcrumb(PreferenceCategoryBreadcrumb):
    """Breadcrumb for /++preferences++/ --> hide."""

    grok.adapts(
        zope.preference.interfaces.IPreferenceGroup,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    show = False


class SystemErrorViewBreadcrumb(Breadcrumb):
    """Breadcrumb to render a title for system error views."""

    grok.adapts(
        zope.browser.interfaces.ISystemErrorView,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    target_url = None
    title = 'SystemError'


class UnauthorizedPageletBreadcrumb(Breadcrumb):
    """Do not render a breadcrumb for the unauthorized pagelet."""

    grok.adapts(
        z3c.layer.pagelet.interfaces.IUnauthorizedPagelet,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    show = False

    @property
    def parent(self):
        return icemac.addressbook.interfaces.IAddressBook(self.context)


class NotFoundPageletBreadcrumb(Breadcrumb):
    """Do not render a breadcrumb for the not found pagelet."""

    grok.adapts(
        z3c.layer.pagelet.interfaces.INotFoundPagelet,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    show = False


class NotFoundBreadcrumb(Breadcrumb):
    """Breadcrumb to render a title for the NotFound exception."""

    grok.adapts(
        zope.publisher.interfaces.INotFound,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    target_url = None
    title = _('Not Found')


class EntityBreadcrumb(Breadcrumb):
    """Breadcrumb for an entity."""

    grok.adapts(
        icemac.addressbook.interfaces.IEntity,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    @property
    def parent(self):
        return zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)


class BreadcrumbContentProvider(
        grok.MultiAdapter,
        zope.contentprovider.provider.ContentProviderBase):
    """Content provider rendering the breadcrumbs."""

    grok.adapter(
        zope.interface.Interface,
        icemac.addressbook.browser.interfaces.IAddressBookLayer,
        zope.interface.Interface)
    grok.provides(zope.contentprovider.interfaces.IContentProvider)
    grok.name('breadcrumbs')

    def render(self):
        return '\n'.join(self.rendered_breadcrumbs)

    @property
    def rendered_breadcrumbs(self):
        for bc in reversed(list(self.reversed_breadcrumbs)):
            yield bc.render()

    @property
    def reversed_breadcrumbs(self):
        bc = self.get_breadcrumb(self.__parent__)
        parent = bc.parent
        while parent is not None:
            if bc.show:
                yield bc
            bc = self.get_breadcrumb(parent)
            parent = bc.parent
        assert not bc.show, 'Found a root breadcrumb with show = True'

    def get_breadcrumb(self, context):
        return zope.component.getMultiAdapter(
            (context, self.request), IBreadcrumb)



from icemac.addressbook.interfaces import ITitle
import icemac.addressbook.interfaces
import zope.interface


def test_adapter__default_title__1(zcmlS):
    """It returns a string unchanged."""
    assert 'asdf' == ITitle('asdf')


def test_adapter__default_title__2(zcmlS):
    """It returns a unicode unchanged."""
    assert u'qwe' == ITitle(u'qwe')


def test_adapter__default_title__3(zcmlS):
    """It returns a the string representation of other objects."""
    assert ITitle(object()).startswith('<object object at 0x')


def test_adapter__exception_title__1(zcmlS):
    """It renders the exception manually instead of using str(exception).

    The latter is overwritten by Chameleon with a currently ugly and lengthy
    representation.
    """
    assert u'<KeyError(42,)>' == ITitle(KeyError(42))


def test_adapter__SchemaProvider_SchemaName__schema_name__1(zcmlS):
    """It is the name of the interface of the schema provider."""
    @zope.interface.implementer(icemac.addressbook.interfaces.ISchemaProvider)
    class Foo(object):
        schema = ITitle

    schema_name = icemac.addressbook.interfaces.ISchemaName(Foo()).schema_name
    assert 'ITitle' == schema_name
    assert isinstance(schema_name, unicode)


def test_adapter__Interface__schema_name__1(zcmlS):
    """It is the name of the interface."""
    schema_name = icemac.addressbook.interfaces.ISchemaName(ITitle).schema_name
    assert 'ITitle' == schema_name
    assert isinstance(schema_name, unicode)



from icemac.addressbook.namechooser.interfaces import IDontReuseNames
from icemac.addressbook.namechooser.interfaces import INameSuffix
from icemac.addressbook.namechooser.namechooser import DontReuseNames
from icemac.addressbook.namechooser.namechooser import NameSuffix
from zope.interface.verify import verifyObject
import mock
import pytest
import six
import zope.annotation.interfaces
import zope.container.sample
import zope.interface


@pytest.yield_fixture(scope='function')
def NameChooserFactory(zcmlS):
    """Get a function to create a name chooser."""
    patchers = []

    def create_name_chooser(name_suffix, patchers=patchers):
        """Create a name chooser with `name_suffix` as current suffix value."""
        nc = DontReuseNames(mock.MagicMock())
        patcher = mock.patch.object(
            DontReuseNames, 'name_suffix', new_callable=mock.PropertyMock)
        name_suffix_mock = patcher.start()
        name_suffix_mock.return_value = name_suffix
        patchers.append(patcher)  # handover the patcher to tear down
        return nc
    yield create_name_chooser
    for patcher in patchers:
        patcher.stop()


def test_namechooser__NameSuffix__1():
    """`NameSuffix` conforms to INameChooser."""
    assert verifyObject(INameSuffix, NameSuffix())


def test_namechooser__NameSuffix__2():
    """`NameSuffix` starts with a value of zero."""
    assert u'0' == six.text_type(NameSuffix())


def test_namechooser__NameSuffix__3():
    """`NameSuffix` sums up."""
    name_suffix = NameSuffix()
    name_suffix += 3
    name_suffix += 2
    assert u'5' == six.text_type(name_suffix)


def test_namechooser__DontReuseNames__chooseName__1(NameChooserFactory):
    """`chooseName()` includes a suffix even for the first name."""
    namechooser = NameChooserFactory(0)
    assert u'foo-1' == namechooser.chooseName('foo', object())


def test_namechooser__DontReuseNames__chooseName__2(NameChooserFactory):
    """`chooseName()` counts the suffix up."""
    namechooser = NameChooserFactory(1)
    assert u'foo-2' == namechooser.chooseName('foo', object())


def test_namechooser__DontReuseNames__chooseName__3(NameChooserFactory):
    """`chooseName()` ignores the name prefix when counting."""
    namechooser = NameChooserFactory(1)
    assert u'bar-2' == namechooser.chooseName('bar', object())


def test_namechooser__DontReuseNames__chooseName__4(NameChooserFactory):
    """`chooseName()` puts the suffix before the extension in the name."""
    namechooser = NameChooserFactory(6)
    assert u'baz-7.txt' == namechooser.chooseName('baz.txt', object())


def test_namechooser__DontReuseNames__chooseName__5(NameChooserFactory):
    """`chooseName()` strips leading + signs from name."""
    namechooser = NameChooserFactory(5)
    assert u'b+z+-6' == namechooser.chooseName('+++b+z+', object())


def test_namechooser__DontReuseNames__chooseName__6(NameChooserFactory):
    """`chooseName()` strips leading @ signs from name."""
    namechooser = NameChooserFactory(5)
    assert u'b@z@-6' == namechooser.chooseName('@@b@z@', object())


def test_namechooser__DontReuseNames__chooseName__7(NameChooserFactory):
    """`chooseName()` replaces slashes with minus signs."""
    namechooser = NameChooserFactory(5)
    assert u'-b-a-z--6' == namechooser.chooseName('/b/a/z/', object())


def test_namechooser__DontReuseNames__chooseName__8(NameChooserFactory):
    """`chooseName()` uses the class name if name is empty after stripping."""
    namechooser = NameChooserFactory(2)
    assert u'object-3' == namechooser.chooseName('+', object())


def test_namechooser__DontReuseNames__chooseName__9(NameChooserFactory):
    """`chooseName()` omits a name alredy used in the container."""
    nc = NameChooserFactory(2)
    with mock.patch.object(nc, 'name_in_use', side_effect=[True, False]):
        assert u'foo-4' == nc.chooseName('foo', object())


def test_namechooser__DontReuseNames__1(zcmlS):
    """`DontReuseNames` stores its data on the container."""
    container = zope.container.sample.SampleContainer()
    zope.interface.alsoProvides(container, IDontReuseNames)
    zope.interface.alsoProvides(
        container, zope.annotation.interfaces.IAttributeAnnotatable)
    nc = zope.container.interfaces.INameChooser(container)
    assert isinstance(nc, DontReuseNames)
    assert u'foo-1' == nc.chooseName('foo', object())
    nc2 = zope.container.interfaces.INameChooser(container)
    assert u'foo-2' == nc2.chooseName('foo', object())



import pytest


@pytest.mark.webdriver
def test_preferences__1_webdriver(address_book, webdriver, FullPersonFactory):
    """Testing JS functions of preferences front-end."""
    FullPersonFactory(address_book, u'Tester')
    prefs = webdriver.prefs
    webdriver.login('mgr', prefs.PREFS_URL)

    # Preference groups are closed by default, fields are not visible:
    prefs.wait_for_fields_visible(False)
    prefs.toggle_group('personLists')
    prefs.wait_for_fields_visible(True)
    # Elements can be removed from the columns list:
    prefs.remove_first_column_selected_for_person_list()

    # New elements can be added:
    prefs.select_column_for_person_list("postal address -- city")
    prefs.select_column_for_person_list("person -- notes")

    # the group can be closed again:
    prefs.toggle_group('personLists')
    prefs.wait_for_fields_visible(False)

    prefs.submit()
    assert 'Data successfully updated.' == webdriver.message

    personlist = webdriver.personlist
    assert ['first name', 'city', 'notes'] == personlist.column_headlines

    # The selected columns are stored and rendered correctly when visiting
    # the form again:
    webdriver.open(prefs.PREFS_URL)
    prefs.wait_for_fields_visible(False)
    prefs.toggle_group('personLists')
    prefs.wait_for_fields_visible(True)
    assert [
        u'person -- first name',
        u'postal address -- city',
        u'person -- notes',
    ] == prefs.selected_columns_for_person_list



# We cannot depend here on something besides the standard library as this is
# also used when installing the dependencies!
try:
    import configparser
except ImportError:
    # Python 2
    import ConfigParser as configparser  # noqa
